using UnityEngine;
using System.Collections;

public class PlayerControl : MonoBehaviour
{
    Vector3 targetPos;

    public float speed = 1;
    public float TurnSpeed = 10.0f;
    public float HitWolfRadius = 4.0f;

    public GameObject HitWolfEffect;
    public GameObject SheepObj;

    public float WolfSpawnDelay = 20.0f;
    float m_fSpawnWolfCountdown = -1.0f;

    public bool AllowControl = true;

    public AudioClip HitSheep;
    public AudioClip HitWolf;
    public AudioClip HittingSound;
    public AudioClip SnedigMusik;
    public AudioClip HappyMusik;

    // Use this for initialization
    void Start()
    {
        targetPos = rigidbody.transform.position;
        AllowControl = true;
        GameObject.Find("Music").audio.Stop();
        GameObject.Find("Music").audio.PlayOneShot(SnedigMusik);
    }

    // Update is called once per frame
    void FixedUpdate()
    {
        if (Input.GetKeyUp("escape"))
        {
            Application.LoadLevel("menu");
        }

        if (m_fSpawnWolfCountdown >= 0.0f)
        {
            m_fSpawnWolfCountdown -= Time.deltaTime;
            if (m_fSpawnWolfCountdown <= 0.0f)
            {
                GameObject herd = GameObject.Find("SheepHerd");
                if (herd)
                {
                    GameObject sheep = Instantiate(SheepObj, Vector3.zero, Quaternion.identity) as GameObject;
                    SheepControl s = sheep.GetComponent<SheepControl>();
                    s.wolf = true;
                    GameObject.Find("Music").audio.Stop();
                    GameObject.Find("Music").audio.PlayOneShot(SnedigMusik);
                }
                m_fSpawnWolfCountdown = -1.0f;
            }
        }
        if (AllowControl && Input.GetButtonDown("Movement"))
        {
            // Construct a ray from the current mouse coordinates
            RaycastHit hitinfo;
            Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
            if (Physics.Raycast(ray, out hitinfo, 1 << 9)) // hit sheep
            {
                Vector3 worldPos = ray.GetPoint(hitinfo.distance);
                SheepControl sheep = hitinfo.collider.GetComponent<SheepControl>();
                if (sheep && (worldPos - transform.position).magnitude < HitWolfRadius)
                {
                    if (sheep.wolf)
                    {
                        audio.PlayOneShot(HitWolf);
                        GameObject.Find("Music").audio.Stop();
                        GameObject.Find("Music").audio.PlayOneShot(HappyMusik, 0.1f);
                        m_fSpawnWolfCountdown = WolfSpawnDelay; // seconds
                    }
                    else
                    {
                        audio.PlayOneShot(HitSheep);
                    }
                    Instantiate(HitWolfEffect, worldPos + Vector3.up * 1.0f, Quaternion.identity);
                    Destroy(sheep.gameObject);
                    return;
                }
            }
        }
        if (AllowControl && Input.GetButton("Movement"))
        {
            // Construct a ray from the current mouse coordinates
            RaycastHit hitinfo;
            Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
            if (Physics.Raycast(ray, out hitinfo, 1 << 8)) // hit ground
            {
                Vector3 worldPos = ray.GetPoint(hitinfo.distance);
                targetPos = worldPos;

                Vector3 dif = (targetPos - rigidbody.transform.position);
                dif.y = 0.0f;
                if (dif.magnitude > speed)
                {
                    dif = dif.normalized * speed;
                }
                Quaternion qVel = Quaternion.Euler(0.0f, 180.0f / Mathf.PI * Mathf.Atan2(-dif.normalized.z, dif.normalized.x), 0.0f);
                Quaternion qLook = rigidbody.transform.rotation;
                rigidbody.rotation = Quaternion.Lerp(qLook, qVel, Time.deltaTime * TurnSpeed);
                rigidbody.AddRelativeForce(new Vector3(speed,0.0f,0.0f));
            }
        }

        if (rigidbody.velocity.magnitude < 1.0f)
        {
            rigidbody.angularVelocity = Vector3.zero;
        }
    }
}
