using UnityEngine;
using System.Collections;

public class SheepControl : MonoBehaviour {

    public float fleeDistance = 5;
    public float fleeSpeed = 3;
    Vector3 targetPos;
    public Vector3 wtf;
//    float movement = 0;
    int movementCounter = 1000;
//    GameObject bounds;
//    GameObject floor;
    Renderer gfx;
    GameObject endZone;
    public float homingFactor = 10;
    public bool wolf = false;
    int wolfTimer = 0;
    bool runToTargetAsWolf=false;
    public float wolfSpeed = 20;
    GameObject huntedSheep = null;
    Renderer sheepArea = null;
    public float centerFactor = 0.7f;
    float m_fLastKiss;

    public GameObject KissEffect;
    public GameObject EatSheepEffect;
    public GameObject SheepObj;

    public GameObject WolfGfx;
    public GameObject SheepGfx;

    public AudioClip[] SheepFlee;
    public AudioClip EatSheepSound;
    public AudioClip AttackingWolfSound;

	// Use this for initialization
	void Start () {
        targetPos = transform.position;
//        bounds = GameObject.Find("bounds");
//        floor = GameObject.Find("floor");
        gfx = GetComponentInChildren<Renderer>();
        endZone = GameObject.Find("EndZone");

        sheepArea = GameObject.FindGameObjectWithTag("SheepArea").GetComponentInChildren<Renderer>();
        m_fLastKiss = -5.0f;

        WolfGfx.active = false;
        SheepGfx.active = true;
    }

    GameObject FindFarthestSheep() {
        GameObject[] sheepList = GameObject.FindGameObjectsWithTag("Sheep");
        float bestDist = 0;
        GameObject bestSheep = null;
        foreach (GameObject go in sheepList)
        {
            if (go == this)
            {
                continue;
            }
            float dist = (go.transform.position - transform.position).magnitude;
            if(dist > bestDist){
                bestDist = dist;
                bestSheep = go;
            }
        }
        return bestSheep;
    }
	
	// Update is called once per frame
	void FixedUpdate () {

        GameObject player = GameObject.FindGameObjectWithTag("Player");
        Vector3 dif = player.transform.position - transform.position;

        bool mouseDown = true;//Input.GetButton("Bark");

        // Sheep Fleeing!
        if (mouseDown && dif.magnitude < fleeDistance && !runToTargetAsWolf)
        {
            Vector3 difToEndZone = transform.position - endZone.transform.position;
            float closeToHome = 1 - Mathf.Min(difToEndZone.magnitude / homingFactor,1); 
            dif = Vector3.Lerp(dif,difToEndZone,closeToHome);
            targetPos = transform.position - (dif.normalized * fleeSpeed);
            targetPos.y = 0;
            movementCounter = 0;
            if (Random.Range(0, 60) == 0 && !audio.isPlaying)
            {
                audio.PlayOneShot(SheepFlee[Random.Range(0,SheepFlee.Length)], 0.4f);
            }
        }
        

        if(movementCounter > 120){
            // Sheep finds random target
            Debug.Log("Sheep goes to middle");
            Bounds area = sheepArea.renderer.bounds;
            area.size = area.size * centerFactor;
            targetPos.x = Random.Range(area.min.x,area.max.x);
            targetPos.z = Random.Range(area.min.z,area.max.z);
            targetPos.y = 0;
            movementCounter = 0;
        }
        movementCounter += 1;

        // Wolf Logic
        if(wolf){
            if (wolfTimer > 170 && wolfTimer < 200)
            {
                float f = Mathf.Sin(wolfTimer) * 0.1f + 1.1f;
                transform.localScale = new Vector3(f, 1.0f, f);
            }
            else if(wolfTimer == 200){
                transform.localScale = new Vector3(1.0f, 1.0f, 1.0f);
                // Transform and run to another place
                //gfx.material.color = new UnityEngine.Color(0.9f, 0.9f, 1, 1);
                WolfGfx.active = true;
                SheepGfx.active = false;
                audio.PlayOneShot(AttackingWolfSound);
                huntedSheep = FindFarthestSheep();
                targetPos = huntedSheep.transform.position;
                runToTargetAsWolf = true;
            }
            else if(wolfTimer == 300){
                //gfx.material.color = new UnityEngine.Color(1, 1, 1, 1);
                WolfGfx.active = false;
                SheepGfx.active = true;
                runToTargetAsWolf = false;
                wolfTimer = 0;
            }
            wolfTimer++;
       }


        dif = (targetPos - rigidbody.transform.position);
        float usedSpeed = fleeSpeed;
        if (runToTargetAsWolf)
            usedSpeed = wolfSpeed;

        if (dif.magnitude > 3.0f)
        {
            dif = dif.normalized * usedSpeed;
            rigidbody.velocity = dif;
        }
        else{
            if (runToTargetAsWolf){
                if(huntedSheep != null){
                    float distFromSheepToWolf = (huntedSheep.transform.position - transform.position).magnitude; 
                    if(distFromSheepToWolf <3.0f) {
                        Instantiate(EatSheepEffect, huntedSheep.transform.position + Vector3.up * 1.0f, Quaternion.identity);
                        audio.PlayOneShot(EatSheepSound);
                        Destroy(huntedSheep);
                    }
                    huntedSheep = null;

                }
                //gfx.material.color = new UnityEngine.Color(1, 1, 1, 1);
                WolfGfx.active = false;
                SheepGfx.active = true;
                runToTargetAsWolf = false;
                wolfTimer = 0;
            }
            rigidbody.velocity = new UnityEngine.Vector3(0, 0, 0);
        }
        dif.y = 0;
        rigidbody.rotation = Quaternion.Euler(0.0f, 180.0f / Mathf.PI * Mathf.Atan2(-dif.normalized.z, dif.normalized.x), 0.0f);
    }


    void OnCollisionEnter(Collision collision)
    {
        if (!wolf && m_fLastKiss + 15.0f < Time.time)
        {
            SheepControl other = collision.gameObject.GetComponent<SheepControl>();
            if (other && !other.wolf)
            {
                Instantiate(KissEffect, transform.position + Vector3.up * 1.0f, Quaternion.identity);
                m_fLastKiss = Time.time;
                if (Random.Range(0, 100) == 0)
                {
                  //  GameObject sheep = Instantiate(SheepObj, collision.transform.position+new Vector3(1.0f,0.0f,1.0f), Quaternion.identity) as GameObject;
                }
            }
        }
    }
}
